---------------------------------------------------------------------------------------
--
--	Author :	Sergio Sanchez
--	Email  :	sergio.sanchez@xilinx.com
--	Phone  :	(408) 879-4637
--	Company:	Xilinx
--
--   Disclaimer:	THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--			WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY
--			IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--			A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--			Copyright (c) 1999 Xilinx, Inc.
--			All rights reserved
--
--	Version:	$Id: ctlr_p.vhd,v 1.1 1999-08-06 sanchez Exp $
--
---------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------
-- This ZBTSRAM controller assumes that only the read/write signal is being used
-- The other control signals are tied inside FPGA logic for future flexibility
-- The user can instead tie them on the board and save on the pins
---------------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
package my_ctlr_p is
	component CTLR_P
	generic(ADDR_BITS : integer := 16;
		DATA_BITS : integer := 36);
	port(
	     dq		  : inout  std_logic_vector(DATA_BITS-1 downto 0);
	     addr	  : out    std_logic_vector(ADDR_BITS-1 downto 0);
	     lbo_n	  : out    std_logic;
	     clk	  : out    std_logic;
	     cke_n	  : out    std_logic;
	     ld_n	  : out    std_logic;
	     bwa_n	  : out    std_logic;
	     bwb_n	  : out    std_logic;
	     bwc_n	  : out    std_logic;
	     bwd_n	  : out    std_logic;
	     rw_n	  : out    std_logic;
	     oe_n	  : out    std_logic;
	     ce_n	  : out    std_logic;
	     ce2	  : out    std_logic;
	     ce2_n	  : out    std_logic;
	     zz		  : out    std_logic;
	     fpga_clk	  : out    std_logic;
	     ui_addr	  : in     std_logic_vector(ADDR_BITS-1 downto 0);
	     ui_write_data: in	   std_logic_vector(DATA_BITS-1 downto 0);
	     ui_rw_n	  : in     std_logic;
	     ui_rw_n_ctlr : in     std_logic;
	     ui_board_clk : in     std_logic;
	     ui_read_data : out    std_logic_vector(DATA_BITS-1 downto 0);
	     ui_clk_mirror_locked_int : out std_logic;
	     ui_clk_mirror_locked_ext : out std_logic;
	     ui_clk_mirror_fb         : in  std_logic);
	end component;
end my_ctlr_p;

library IEEE;
library unisim;
use unisim.vcomponents.all;
use IEEE.std_logic_1164.all;

entity CTLR_P is
	generic(ADDR_BITS : integer := 16;
		DATA_BITS : integer := 36);
	port(
	     dq		  : inout  std_logic_vector(DATA_BITS-1 downto 0);--bidirectional data bus to RAM
	     addr	  : out    std_logic_vector(ADDR_BITS-1 downto 0);--addr goes out to the RAM
	     lbo_n	  : out    std_logic;  --Burst Mode (0 = Linear, 1 = Interleaved)
	     clk	  : out    std_logic;  --2x clock goes to the RAM
	     cke_n	  : out    std_logic;  --Synchronous Clock Enable
	     ld_n	  : out    std_logic;  --Synchronous address Adv/LD
	     bwa_n	  : out    std_logic;  --Synchronous Byte Write Enable A
	     bwb_n	  : out    std_logic;  --Synchronous Byte Write Enable B
	     bwc_n	  : out    std_logic;  --Synchronous Byte Write Enable C
	     bwd_n	  : out    std_logic;  --Synchronous Byte Write Enable D
	     rw_n	  : out    std_logic;  --Read Write control goes out to the RAM
	     oe_n	  : out    std_logic;  --Output Enable
	     ce_n	  : out    std_logic;  --Synchronous Chip Enable
	     ce2	  : out    std_logic;  --Synchronous Chip Enable
	     ce2_n	  : out    std_logic;  --Synchronous Chip Enable
	     zz		  : out    std_logic;  --Snooze Mode
	     fpga_clk	  : out    std_logic;  --clk to use inside fpga (= ui_board_clk * 2)

	     ui_addr	  : in     std_logic_vector(ADDR_BITS-1 downto 0);--addr comes in from the user design
	     ui_write_data: in	   std_logic_vector(DATA_BITS-1 downto 0);--data comes in from the user design
	     ui_rw_n	  : in     std_logic;  --Read write control comes from the user design
	     ui_rw_n_ctlr : in     std_logic;  --Read write control comes from the user design
	     ui_board_clk : in     std_logic;  --incoming clock into the Virtex CLKDLL
	     ui_read_data : out    std_logic_vector(DATA_BITS-1 downto 0);--data read from SRAM
	     ui_clk_mirror_locked_int : out std_logic;
	     ui_clk_mirror_locked_ext : out std_logic;
	     ui_clk_mirror_fb         : in  std_logic); --2x clock feedback from the ui_board
end CTLR_P;

architecture RTL of CTLR_P is
signal write_data	: std_logic_vector(DATA_BITS-1 downto 0);
signal read_data	: std_logic_vector(DATA_BITS-1 downto 0);
signal rw_tff		: std_logic_vector(DATA_BITS-1 downto 0);
signal rw_n_p		: std_logic;
signal fpga_clk_a	: std_logic;

component DATABITS_INOUT
	port(
	     read_data 		: out   std_logic_vector(DATA_BITS-1 downto 0);
	     dq			: inout std_logic_vector(DATA_BITS-1 downto 0);
	     write_data		: in    std_logic_vector(DATA_BITS-1 downto 0);
	     rw_tff		: in    std_logic_vector(DATA_BITS-1 downto 0);
	     fpga_clk		: in    std_logic);
end component;
 
component PIPELINED_STAGES_P 
	port(
	     ui_read_data	: out   std_logic_vector(DATA_BITS-1 downto 0);
	     write_data		: out   std_logic_vector(DATA_BITS-1 downto 0);
	     rw_tff		: out   std_logic_vector(DATA_BITS-1 downto 0);
	     ui_write_data    	: in    std_logic_vector(DATA_BITS-1 downto 0);
 	     ui_rw_n		: in    std_logic;
	     read_data		: in    std_logic_vector(DATA_BITS-1 downto 0);
	     fpga_clk		: in    std_logic);
end component;

component ADDRBITS_OUT
	port(
	     addr	: out std_logic_vector(ADDR_BITS-1 downto 0);
	     ui_addr	: in  std_logic_vector(ADDR_BITS-1 downto 0);
	     fpga_clk	: in  std_logic);
end component;

component OBUF_F_16
	port(I : in std_logic; O : out std_logic);
end component;

component TIE_UNUSED_SIGS
	port(
	     lbo_n, cke_n, ld_n	: out std_logic;
	     bwa_n, bwb_n, bwc_n: out std_logic;
	     bwd_n, oe_n, ce_n	: out std_logic;
	     ce2, ce2_n, zz	: out std_logic);
end component; 

component DLL_2X_BOARD
	port(
	     clk2x_int			: out std_logic;
	     ui_clk_mirror_locked_int	: out   std_logic;
	     clk2x_ext			: out   std_logic;
	     ui_clk_mirror_locked_ext	: out   std_logic;
	     ui_board_clk		: in    std_logic;
	     ui_clk_mirror_fb		: in    std_logic);
end component;

begin

---------------------------------------------------------------------------------------
-- write data and read/write has to go thru two levels of pipelining for the 
-- Pipelined ZBT SRAM and one level for the Flowthru ZBT SRAM
-- This module provides appropriate number of pipeline stages
-- and fans out the read/write signal using a tree of FFs
---------------------------------------------------------------------------------------

I_pipelined_stages_p: PIPELINED_STAGES_P port map(
	     					  ui_read_data=>ui_read_data,
	     					  write_data=>write_data,
	    					  rw_tff=>rw_tff,
					  	  ui_write_data=>ui_write_data,
 	     					  ui_rw_n=>ui_rw_n_ctlr,
	     					  read_data=>read_data,
	     					  fpga_clk=>fpga_clk_a);

I_databits_inout: DATABITS_INOUT port map(
	     				  read_data=>read_data,
	     				  dq=>dq,
	     				  write_data=>write_data,
	     				  rw_tff=>rw_tff,
	     				  fpga_clk=>fpga_clk_a);

I_addrbits_out: ADDRBITS_OUT port map( 
	     			      addr=>addr,
	     			      ui_addr=>ui_addr,
	     			      fpga_clk=>fpga_clk_a);

---------------------------------------------------------------------------------------
-- The read/write control signal goes directly to the ZBT SRAM
---------------------------------------------------------------------------------------

	process(fpga_clk_a) begin
		if (fpga_clk_a 'event and fpga_clk_a = '1') then
			rw_n_p <= ui_rw_n;
		end if;
	end process;

I_obuf0: OBUF_F_16 port map(I=>rw_n_p, O=>rw_n);

I_tie_unused_sigs: TIE_UNUSED_SIGS port map(
	     				    lbo_n=>lbo_n, cke_n=>cke_n, ld_n=>ld_n,
	     				    bwa_n=>bwa_n, bwb_n=>bwb_n, bwc_n=>bwc_n,
	     				    bwd_n=>bwd_n, oe_n=>oe_n, ce_n=>ce_n,
	     				    ce2=>ce2, ce2_n=>ce2_n, zz=>zz);

I_dll_2x_board: DLL_2X_BOARD port map(
	     			      clk2x_int=>fpga_clk_a,
	     			      ui_clk_mirror_locked_int=>ui_clk_mirror_locked_int,
	     			      clk2x_ext=>clk,
	     			      ui_clk_mirror_locked_ext=>ui_clk_mirror_locked_ext,
	     			      ui_board_clk=>ui_board_clk,
				      ui_clk_mirror_fb=>ui_clk_mirror_fb);

fpga_clk <= fpga_clk_a;

end RTL;
----------------------------------------------------------------------------------
--
--	Author :	Sergio Sanchez
--	Email  :	sergio.sanchez@xilinx.com
--	Phone  :	(408) 879-4637
--	Company:	Xilinx
--
--   Disclaimer:	THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--			WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY
--			IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--			A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--			Copyright (c) 1999 Xilinx, Inc.
--			All rights reserved
--
--	Version:	$Id: addrbits_out.vhd,v 1.1 1999-08-05 sanchez Exp $
--
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Module for interfacing address with the ZBTSRAM
----------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
library unisim;
use unisim.vcomponents.all;

entity ADDRBITS_OUT is
	generic(ADDR_BITS :integer :=16);
	port(
-- Interfaces to user logic
	     ui_addr : in  std_logic_vector(ADDR_BITS-1 downto 0);
	     fpga_clk: in  std_logic;
-- Interface to ZBT SRAM
	     addr    : out std_logic_vector(ADDR_BITS-1 downto 0));
end ADDRBITS_OUT;

architecture RTL of ADDRBITS_OUT is
signal addr_p : std_logic_vector(ADDR_BITS-1 downto 0);

component OBUF_F_16
	port( I : in  std_logic;
	      O : out std_logic);
end component;

begin

	process(fpga_clk) begin
		if (fpga_clk 'event and fpga_clk = '1') then
			for i in 0 to ADDR_BITS-1 loop
				addr_p(i) <= ui_addr(i);
			end loop;
		end if;
	end process;

g0:	for i in 0 to ADDR_BITS-1 generate

	 obuf_addr: OBUF_F_16 port map(I=>addr_p(i), O=>addr(i));

	end generate;
	
end RTL;
---------------------------------------------------------------------------------------
--
--	Author :	Sergio Sanchez
--	Email  :	sergio.sanchez@xilinx.com
--	Phone  :	(408) 879-4637
--	Company:	Xilinx
--
--   Disclaimer:	THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--			WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY
--			IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--			A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--			Copyright (c) 1999 Xilinx, Inc.
--			All rights reserved
--
--	Version:	$Id: databits_inout.vhd,v 1.1 1999-08-05 sanchez Exp $
--
---------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------
-- Module for interfacing data with the ZBTSRAM
---------------------------------------------------------------------------------------

library IEEE;
library unisim;
use unisim.vcomponents.all;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity DATABITS_INOUT is 
	generic(DATA_BITS : integer := 36);
	port(
-- Interface to DATABITS_INOUT
	     read_data	: out   std_logic_vector(DATA_BITS-1 downto 0);
	     write_data	: in	std_logic_vector(DATA_BITS-1 downto 0);
	     rw_tff	: in 	std_logic_vector(DATA_BITS-1 downto 0);
	     fpga_clk	: in 	std_logic;
-- Interface to ZBT SRAM
	     dq		: inout std_logic_vector(DATA_BITS-1 downto 0));
end DATABITS_INOUT;

architecture RTL of DATABITS_INOUT is
signal write_data_p : std_logic_vector(DATA_BITS-1 downto 0);
signal rw_tff_p     : std_logic_vector(DATA_BITS-1 downto 0);

component IOBUF_F_16
	port(O : out  	std_logic;
	     IO: inout 	std_logic;
	     I : in 	std_logic;
	     T : in  	std_logic);
end component;

begin

	process(fpga_clk)
	begin
		if (fpga_clk 'event and fpga_clk = '1') then
		    for i in 0 to DATA_BITS-1 loop
		      write_data_p(i) <= write_data(i);
		      rw_tff_p(i)     <= rw_tff(i);
                    end loop;
		end if;
       end process;

g0:	for i in 0 to DATA_BITS-1 generate

		iobuf:   IOBUF_F_16 port map(O=>read_data(i),IO=>dq(i),I=>write_data_p(i),T=>rw_tff_p(i));

        end generate;
end RTL;
--------------------------------------------------------------------------------
--
--	Author :	Sergio Sanchez
--	Email  :	sergio.sanchez@xilinx.com
--	Phone  :	(408) 879-4637
--	Company:	Xilinx
--
--   Disclaimer:	THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--			WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY
--			IMPLIED WARRANTIES OF MERCHANTIBILITY, FITNESS FOR
--			A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--			Copyright (c) 1999 Xilinx, Inc.
--			All rights reserved
--
--	Version:	$Id: tie_unused_sigs.vhd,v 1.1 1999-08-05 sanchez Exp $
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--Module for tying the unused control signals
--------------------------------------------------------------------------------

library IEEE;
library unisim;
use unisim.vcomponents.all;
use IEEE.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Interface to ZBT SRAM
--------------------------------------------------------------------------------

entity TIE_UNUSED_SIGS is
	port(
	     lbo_n	: out std_logic; --Burst Mode (0=Linear, 1=Interleaved)
	     cke_n	: out std_logic; --Synchronous Clock Enable
	     ld_n	: out std_logic; --Synchronous address Adv/LD
	     bwa_n	: out std_logic; --Synchronous Byte Write Enable A
	     bwb_n	: out std_logic; --Synchronous Byte Write Enable B
	     bwc_n	: out std_logic; --Synchronous Byte Write Enable C
	     bwd_n	: out std_logic; --Synchronous Byte Write Enable D
	     oe_n	: out std_logic; --OUt Enable
	     ce_n	: out std_logic; --Synchronous Chip Enable
	     ce2	: out std_logic; --Synchronous Chip Enable
	     ce2_n	: out std_logic; --Synchronous Chip Enable
	     zz		: out std_logic);--Snooze Mode
end TIE_UNUSED_SIGS;

architecture RTL of TIE_UNUSED_SIGS is
signal lbo_n_w	: std_logic;
signal cke_n_w	: std_logic;
signal ld_n_w	: std_logic;
signal bwa_n_w	: std_logic;
signal bwb_n_w	: std_logic;
signal bwc_n_w	: std_logic;
signal bwd_n_w	: std_logic;
signal oe_n_w	: std_logic;
signal ce_n_w	: std_logic;
signal ce2_w	: std_logic;
signal ce2_n_w	: std_logic;
signal zz_w	: std_logic;

component OBUF
	port(I : in std_logic; O : out std_logic);
end component;

begin

lbo_n_w	<= '0';
cke_n_w	<= '0';
ld_n_w	<= '0';
bwa_n_w	<= '0';
bwb_n_w	<= '0';
bwc_n_w	<= '0';
bwd_n_w	<= '0';
oe_n_w	<= '0';
ce_n_w	<= '0';
ce2_w	<= '1';
ce2_n_w	<= '0';
zz_w	<= '0';

I_obuf_lbo_n: OBUF port map(I=>lbo_n_w, O=>lbo_n);
I_obuf_cke_n: OBUF port map(I=>cke_n_w, O=>cke_n);
I_obuf_ld_n:  OBUF port map(I=>ld_n_w, O=>ld_n);
I_obuf_bwa_n: OBUF port map(I=>bwa_n_w, O=>bwa_n);
I_obuf_bwb_n: OBUF port map(I=>bwb_n_w, O=>bwb_n);
I_obuf_bwc_n: OBUF port map(I=>bwc_n_w, O=>bwc_n);
I_obuf_bwd_n: OBUF port map(I=>bwd_n_w, O=>bwd_n);
I_obuf_oe_n:  OBUF port map(I=>oe_n_w, O=>oe_n);
I_obuf_ce_n:  OBUF port map(I=>ce_n_w, O=>ce_n);
I_obuf_ce2:   OBUF port map(I=>ce2_w, O=>ce2);
I_obuf_ce2_n: OBUF port map(I=>ce2_n_w, O=>ce2_n);
I_obuf_zz:    OBUF port map(I=>zz_w, O=>zz);

end RTL;
----------------------------------------------------------------------------------------------
--
--	Author :	Sergio Sanchez
--	Email  :	sergio.sanchez@xilinx.com
--	Phone  :	(408) 879-4637
--	Company:	Xilinx
--
--   Disclaimer:	THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--			WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY
--			IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--			A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--			Copyright (c) 1999 Xilinx, Inc.
--			All rights reserved
--
--	Version:	$Id: pipelined_stages_zbtp.vhd,v 1.1 1999-08-04 sanchez Exp $
--
----------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------
-- Customizes the Controller for the ZBT SRAM for the Pipelined variety
----------------------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity PIPELINED_STAGES_P is
	generic(DATA_BITS	: integer := 36);
	port(
	     ui_read_data	: out std_logic_vector(DATA_BITS-1 downto 0);
	     write_data		: out std_logic_vector(DATA_BITS-1 downto 0);
	     rw_tff		: out std_logic_vector(DATA_BITS-1 downto 0);
	     read_data		: in  std_logic_vector(DATA_BITS-1 downto 0);
	     ui_write_data	: in  std_logic_vector(DATA_BITS-1 downto 0);
	     ui_rw_n		: in  std_logic;
	     fpga_clk		: in  std_logic);
end PIPELINED_STAGES_P;

architecture RTL of PIPELINED_STAGES_P is
signal rw_p1		: std_logic_vector(3 downto 0);
signal rw_p2		: std_logic_vector(3 downto 0);
signal rw_p3		: std_logic_vector(3 downto 0);
signal write_data_p	: std_logic_vector(DATA_BITS-1 downto 0);

begin

	process (fpga_clk)
	begin
		if (fpga_clk 'event and fpga_clk = '1') then
----------------------------------------------------------------------------------------------
-- a tree of FFs to reduce fanout and add two pipelining stages
----------------------------------------------------------------------------------------------
			for i in 0 to 3 loop
				rw_p1(i)    <= ui_rw_n;
				rw_p2(i)    <= rw_p1(i);
				rw_p3(i)    <= rw_p2(i);
				rw_tff(i)   <= rw_p1(i);
				rw_tff(i+4) <= rw_p1(i);
				rw_tff(i+8) <= rw_p1(i);
				rw_tff(i+12)<= rw_p1(i);
				rw_tff(i+16)<= rw_p1(i);
				rw_tff(i+20)<= rw_p1(i);
				rw_tff(i+24)<= rw_p1(i);
				rw_tff(i+28)<= rw_p1(i);
				rw_tff(i+32)<= rw_p1(i);
			end loop;
	
----------------------------------------------------------------------------------------------
-- add two pipeline stages to data being written to the ZBT SRAM 
----------------------------------------------------------------------------------------------
			for i in 0 to DATA_BITS-1 loop
				write_data_p(i) <= ui_write_data(i);
				write_data(i)    <= write_data_p(i);
			end loop;
			
----------------------------------------------------------------------------------------------
-- register the data being read back from the ZBT SRAM 
----------------------------------------------------------------------------------------------
			if (rw_p3(0) = '1') then 
				for i in 0 to 8 loop
					ui_read_data(i) <= read_data(i);
				end loop;
			end if;
			if (rw_p3(1) = '1') then 
				for i in 9 to 17 loop
					ui_read_data(i) <= read_data(i);
				end loop;
			end if;
			if (rw_p3(2) = '1') then 
				for i in 18 to 26 loop
					ui_read_data(i) <= read_data(i);
				end loop;
			end if;
			if (rw_p3(3) = '1') then 
				for i in 27 to 35 loop
					ui_read_data(i) <= read_data(i);
				end loop;
			end if;
								
		end if;
	end process;
end RTL;
-----------------------------------------------------------------------------
--
--	Author :	Sergio Sanchez
--	Email  :	sergio.sanchez@xilinx.com
--	Phone  :	(408) 879-4637
--	Company:	Xilinx
--
--   Disclaimer:	THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--			WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY
--			IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--			A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--			Copyright (c) 1999 Xilinx, Inc.
--			All rights reserved
--
--	Version:	$Id: dll_2x_board.vhd,v 1.1 1999-08-05 sanchez Exp $
--
-----------------------------------------------------------------------------

-----------------------------------------------------------------------------
--This module instantiates two DLLs for generating de-skewed 2X clocks
--one for use inside the fpga (_int)
--and one for interfacing with other components on the Board (_ext)
-----------------------------------------------------------------------------

library IEEE;
library unisim;
use unisim.vcomponents.all;
use IEEE.std_logic_1164.all;

entity DLL_2X_BOARD is 
	port(
	     ui_board_clk		: in  std_logic; --Incoming clock
	     ui_clk_mirror_fb		: in  std_logic; --Feedback signal for the external clock
							 --user has to provide connection on the board
	     clk2x_int			: out std_logic; --2X clk available inside the fpga
	     ui_clk_mirror_locked_int	: out std_logic; --locked signal for the internal clock
	     clk2x_ext			: out std_logic; --2X clk going out of the fpga
	     ui_clk_mirror_locked_ext	: out std_logic);--locked signal for the external clock
end DLL_2X_BOARD;

architecture RTL of DLL_2X_BOARD is
signal logic0			: std_logic;
signal clkin			: std_logic;
signal clk_mirror_fb_ext	: std_logic;
signal clk2xdll_int		: std_logic;
signal clk2xdll_ext		: std_logic;
signal clk_mirror_locked_ext	: std_logic;
signal clk_mirror_locked_ext_inv: std_logic;
signal clk_mirror_locked_int	: std_logic;
signal clk_mirror_locked_int_inv: std_logic;
signal clk2x_inta		: std_logic;

component IBUFG
	port(I : in std_logic; O : out std_logic);
end component;

component CLKDLL
	port(clkin : in std_logic;
	     clkfb : in std_logic;
	     rst   : in std_logic;
	     locked: out std_logic;
	     clk2x : out std_logic);
end component;

component BUFG
	port(I : in std_logic; O : out std_logic);
end component;

component OBUF_F_16 
	port(I : in std_logic; O : out std_logic);
end component;

component OBUF_F_8
	port(I : in std_logic; O : out std_logic);
end component;

begin

logic0 <= '0'; -- Never reset the DLL

I_ibufg_clkin: IBUFG port map(I=>ui_board_clk, O=>clkin);

I_clkdll_int : CLKDLL port map(clkin=>clkin, clkfb=>clk2x_inta, rst=>logic0, clk2x=>clk2xdll_int, locked=>clk_mirror_locked_int);

I_bufg_clk2x_int: BUFG port map(I=>clk2xdll_int, O=>clk2x_inta);

clk2x_int <= clk2x_inta;

I_clkdll_ext : CLKDLL port map(clkin=>clkin, clkfb=>clk_mirror_fb_ext, rst=>logic0, clk2x=>clk2xdll_ext, locked=>clk_mirror_locked_ext);

clk_mirror_locked_int_inv <= not clk_mirror_locked_int;

I_obuf_clk_mirror_locked_int: OBUF_F_16 port map(I=>clk_mirror_locked_int_inv, O=>ui_clk_mirror_locked_int);

I_obuf_clk2x_ext: OBUF_F_8 port map(I=>clk2xdll_ext, O=>clk2x_ext);

I_ibufg_clk_mirror_fb_ext: IBUFG port map(I=>ui_clk_mirror_fb, O=>clk_mirror_fb_ext);

clk_mirror_locked_ext_inv <= not clk_mirror_locked_ext;

I_obuf_clk_mirror_locked_ext: OBUF_F_16 port map(I=>clk_mirror_locked_ext_inv, O=>ui_clk_mirror_locked_ext);

end RTL;
